AC_DEFUN([BUNDLE_INIT],
[
  AC_ARG_WITH([bundled-packages],
    [  --with-bundled-packages=PKGS For bundle purpose only ],
    [BUNDLED=$withval],
    [BUNDLED=]
  )
  AC_ARG_WITH([bundle-inst-dir],
    [  --with-inst-dir=PKGS For bundle purpose only ],
    [BUNDLE_INST_DIR=$withval/
     if test x"$PKG_CONFIG_PATH" = x; then
       PKG_CONFIG_PATH="${withval}/pkgconfig"
     else
       PKG_CONFIG_PATH="${withval}/pkgconfig:${PKG_CONFIG_PATH}"
     fi
     export PKG_CONFIG_PATH],
    [BUNDLE_INST_DIR=]
  )

  # check for gcc < 4.x
  AC_LANG_PUSH([C])
  AC_CACHE_CHECK([whether gcc is not too recent],
    [tfs_cv_gcc_version],
    [AC_COMPILE_IFELSE(
      [AC_LANG_SOURCE(
        [
@%:@if (__GNUC__ >= 4)
@%:@ error "doesn't work with gcc >= 4.0."
@%:@endif
        ])],
      [tfs_cv_gcc_version=yes],
      [tfs_cv_gcc_version=no])
    test x$tfs_cv_gcc_version != xyes && AC_MSG_ERROR(
      [please downgrade to gcc < 4.0])
  ])
  AC_LANG_POP


  # eval-* doesn't accept -O2, but it is added by default to CFLAGS
  # whithout any option to remove it later. So transfer CFLAGS to
  # AM_CFLAGS
  AM_CFLAGS=$CFLAGS
  CFLAGS_EVAL=$(echo $CFLAGS | sed -e 's/-O2/-O1/')
  CFLAGS=
  AC_SUBST([AM_CFLAGS])
  AC_SUBST([CFLAGS_EVAL])

  # Create spec file.
  AC_CONFIG_COMMANDS([AC_PACKAGE_TARNAME.spec], 
  [
    TFS_VERSION=$PACKAGE_VERSION \
    TFS_INSTALLED_DIR="$TFS_INSTALLED_DIR" \
    TFS_PKG_DEPS="$TFS_PKG_DEPS" \
    srcdir=$srcdir \
    $srcdir/config/make-spec.sh > rpm-package.sed

    sed -f rpm-package.sed > AC_PACKAGE_TARNAME.spec <<EOF
Summary: Transformers - $PACKAGE_TARNAME
Name: $PACKAGE_TARNAME
Version: $PACKAGE_VERSION
Vendor: LRDE
Release: 1
License: LGPL
Group: Development/Languages/Transformers
URL: https://transformers.lrde.epita.fr
Source: $PACKAGE_TARNAME-$PACKAGE_VERSION.tar.gz
BuildRoot: %{_tmppath}/%{name}-$PACKAGE_VERSION-buildroot
Requires: strategoxt >= 0.14

%description

%prep
%setup -q

%build
./configure --prefix=%{_prefix}
make

%install
rm -rf \$RPM_BUILD_ROOT
make DESTDIR=\$RPM_BUILD_ROOT install

%clean
rm -rf \$RPM_BUILD_ROOT

%files
%defattr(-,root,root,-)

%changelog
* Mon Oct 21 2005 Olivier Gournet <olivier.gournet@lrde.epita.fr>
- Initial build.
EOF
    rm -f rpm-package.sed
  ], [
    TFS_INSTALLED_DIR="$TFS_INSTALLED_DIR"
    TFS_PKG_DEPS="$TFS_PKG_DEPS"
    PACKAGE_TARNAME="$PACKAGE_TARNAME"
    PACKAGE_VERSION="$PACKAGE_VERSION"
  ])

])

AC_DEFUN([BUNDLE_REVISION],
[
  XT_SVN_REVISION

  # comment to make this a stable release
  #XT_PRE_RELEASE
])

AC_DEFUN([BUNDLE_CHECK_PACKAGE],
[
  case "$BUNDLED" in
    *"$2"*) XT_CHECK_PACKAGE([$1], [$2]) ;;
    *) XT_CHECK_PACKAGE([$1], [$2], [$3]) ;;
  esac

  TFS_PKG_DEPS="$TFS_PKG_DEPS:$2" 
  AC_SUBST([TFS_PKG_DEPS])
])

AC_DEFUN([IF_IN_BUNDLE], [ echo "$BUNDLED" | grep -q "$1" ])
