/*
 * attribute.cc: this file is a part of sdf-attribute from the
 *               Transformers project.
 *
 * Copyright (C) 2005  EPITA Research and Development Laboratory.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "checker.hh"
#include "attribute.hh"
#include "production.hh"

Attribute::Attribute(const std::string& ns, const std::string& attr_name)
  : is_root(true),
    symbol_name("<error>"),
    label_name("<error>"),
    ns(ns),
    attribute_name(attr_name)
{
}

Attribute::Attribute(const std::string& symb_name, const std::string& label_name,
		     const std::string& ns, const std::string& attr_name)
  : is_root(false),
    symbol_name(symb_name),
    label_name(label_name),
    ns(ns),
    attribute_name(attr_name)
{
}

Attribute::Attribute(ATerm a, Production& n)
  : is_root(false),
    symbol_name("<error>"),
    label_name("<error>")
{
  ATerm sym, ns, name;
  if (ATmatch(a, "attribute(Sym(<term>), Some(namespace-head(namespace-name(<term>))), <term>)",
	      &sym, &ns, &name) == ATtrue)
    {
      this->label_name = get_string(sym);
      this->symbol_name = n.labelToSymbol(get_string(sym));
    }
  else if (ATmatch(a, "attribute(root, Some(namespace-head(namespace-name(<term>))), <term>)",
		   &ns, &name) == ATtrue)
    {
      is_root = true;
    }
  else
    {
      ATprintf("can't parse attribute: %t\n", a);
      exit(1);
    }

  this->ns = get_string(ns);
  this->attribute_name = get_string(name);
}


std::ostream& operator<< (std::ostream& os, const Attribute& a)
{
  os << (a.is_root ? "root" : a.label_name) << "." << a.ns << ":" << a.attribute_name;
  return os;
}
