/*
 * attribute.hh: this file is a part of sdf-attribute from the
 *               Transformers project.
 *
 * Copyright (C) 2005  EPITA Research and Development Laboratory.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef ATTRIBUTE_HH_
# define ATTRIBUTE_HH_

# include <sstream>

class Attribute
{
public:
  Attribute(const std::string& ns, const std::string& attr_name);
  Attribute(const std::string& symb_name, const std::string& label_name,
	    const std::string& ns, const std::string& attr_name);
  Attribute(ATerm a, Production& n);

  friend std::ostream& operator<< (std::ostream& os, const Attribute& a);

  const std::string& getSymbolName() const;
  const std::string& getLabelName() const;
  const std::string& getNs() const { return ns; }
  const std::string& getAttrName() const { return attribute_name; }
  bool isRoot() const { return is_root; }

  void setAttrName(const std::string& nn ) { attribute_name = nn; }
  void setRoot(bool b) { is_root = b; }

  friend bool operator < (const Attribute& lhs, const Attribute& rhs)
  {
    // trim "rule_"
    std::istringstream l(lhs.label_name.substr(5, 999));
    std::istringstream r(rhs.label_name.substr(5, 999));
    int il, ir;
    l >> il;
    r >> ir;
    return il < ir;
  }

  friend bool operator <= (const Attribute& lhs, const Attribute& rhs)
  {
    // trim "rule_"
    std::istringstream l(lhs.label_name.substr(5, 999));
    std::istringstream r(rhs.label_name.substr(5, 999));
    int il, ir;
    l >> il;
    r >> ir;
    return il <= ir;
  }

  friend bool operator == (const Attribute& lhs, const Attribute& rhs)
  {
    if (lhs.is_root)
      return lhs.is_root == rhs.is_root && lhs.ns == rhs.ns && lhs.attribute_name == rhs.attribute_name;
    return lhs.is_root == rhs.is_root && lhs.ns == rhs.ns && lhs.attribute_name == rhs.attribute_name
      && lhs.symbol_name == rhs.symbol_name && lhs.label_name == rhs.label_name;
  }

private:
  bool		is_root;
  std::string	symbol_name;
  std::string	label_name;
  std::string	ns;
  std::string	attribute_name;
};

inline const std::string& Attribute::getSymbolName() const
{
  if (is_root)
    {
      log_0 << "Error: can't retrieve symbol name for root."
	   << ns << ":" << attribute_name << " attribute !" << std::endl;
    }
  return symbol_name;
}

inline const std::string& Attribute::getLabelName() const
{
  if (is_root)
    {
      log_0 << "Error: can't retrieve label name for root."
	   << ns << ":" << attribute_name << " attribute !" << std::endl;
    }
  return label_name;
}


#endif /* !ATTRIBUTE_HH_ */
