/*
 * checker.hh: this file is a part of sdf-attribute from the
 *             Transformers project.
 *
 * Copyright (C) 2005  EPITA Research and Development Laboratory.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef CHECKER_HH_
# define CHECKER_HH_

# include <cassert>
# include <string>
# include <map>
# include <vector>
# include <iostream>
# include <aterm1.h>
# include <aterm2.h>

inline std::string get_string(ATerm a, const char* suffix = "")
{
  std::string r(ATwriteToString(a));
  return r.substr(1, r.length() - 2) + suffix;
}

inline unsigned hashFct(const std::string& str)
{
  unsigned h, g, i;
  unsigned key_len = str.length();

  h = 0;
  for (i = 0; i < key_len; i++)
    {
      h = (h << 4) + str[i];
      if ((g = (h & 0xF0000000)))
	{
	  h ^= (g >> 24);
	  h ^= g;
	}
    }
  return h;
}

extern std::ostream log_0, log_1, log_2, log_3, log_4, log_5, log_6;

// Forward declarations
class Grammar;
class Production;
class EvalRule;

class Checker
{
public:
  Checker();
  static int check(Grammar& g, bool compat);

private:
  int	removeNonCommitedRules(Production& p);
  int	checkProduction(Production& p);
  int	buildDependencyGraph(Production& p);
  int	computeDependencyGraph(Grammar& g);
  int	checkCycle(Production& p);
  int	inh_dep_error;
  int	syn_dep_error;
  int	multiple_def_error;
  int	cycle_error;
};

#endif /* !CHECKER_HH_ */
