/*
 * evalrule.hh: this file is a part of sdf-attribute from the
 *              Transformers project.
 *
 * Copyright (C) 2005  EPITA Research and Development Laboratory.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef EVALRULE_HH_
# define EVALRULE_HH_

# include "attribute.hh"

class EvalRule
{
  friend class Grammar;

public:
  EvalRule(ATerm t, Production& p);
  EvalRule(Attribute attr);

  Attribute left;
  ATerm aterm; /* original aterm: term(attribute-rule(_,_,_)) */
  ATerm aterm_code; /* original aterm for the right hand side of the ':=' */
  ATerm aterm_left;

  typedef std::vector<Attribute> AttributeList;
  AttributeList inh_dep; /* begin with root. */
  AttributeList syn_dep; /* begin with NT. */

  void find_dependencies(ATerm code, Production& n);

  void addDependency(const Attribute& dep);
  void confirmDependency(Production& p);

  bool isQuery() const { return query; }
  void setQuery(bool b) { query = b; }

  bool isAdded() const { return added; }
  void setAdded() { added = true; query = true; }

  bool isConfirmed() const { return confirmed; }

  const Attribute* getSynDep() const;
  const Attribute* getInhDep() const;

  friend std::ostream& operator<< (std::ostream& os, const EvalRule& er);
  friend bool operator < (const EvalRule& lhs, const EvalRule& rhs);

  static std::string cur_name;

private:
  bool query;
  bool added;
  bool confirmed;
};


#endif /* !EVALRULE_HH_ */
