/*
 * graph.hh: this file is a part of sdf-attribute from the Transformers project.
 *
 * Copyright (C) 2005  EPITA Research and Development Laboratory.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef GRAPH_HH_
# define GRAPH_HH_

# include "attribute.hh"

/*
** Data structures needed by the checker, to build and use
** a dependency graph.
*/
class AttrNode
{
public:
  AttrNode(Attribute* a);
  ~AttrNode();

  unsigned getUid() const { return uid_; }
  unsigned getEid() const { return eid_; }
  Attribute* getAttr() const { return attr_; }
  bool isUp() const { return isUp_; }

  bool addLink(AttrNode& to);
  bool removeLink(AttrNode& to);

  friend bool operator== (const AttrNode& lhs, const AttrNode& rhs)
  {
    return lhs.uid_ == rhs.uid_;
  }
  friend std::ostream& operator<< (std::ostream& os, const AttrNode& a);

//private:
  unsigned eid_;
  unsigned uid_;
  Attribute* attr_;
  bool isUp_;

  unsigned* in_;
  unsigned* out_;
  unsigned nbIn_;
  unsigned nbOut_;
  unsigned mIn_;
  unsigned mOut_;
};


typedef std::map<unsigned, AttrNode>::iterator AttrNodeIter;
typedef std::map<unsigned, AttrNode>::const_iterator AttrNodeConstIter;

class AttrNodeList : public std::map<unsigned, AttrNode>
{
private:
  AttrNodeList& operator=(const AttrNodeList&);

public:
  AttrNodeList();
  AttrNodeList(const AttrNodeList&);

  AttrNode&	addAttrNode(Attribute* attr);
  void		mergeList(const AttrNodeList& l);

  friend std::ostream& operator<< (std::ostream& os, const AttrNodeList& l);
};

#endif /* !GRAPH_HH_ */
