/*
 * graphic.cc: this file is a part of sdf-attribute from the
 *             Transformers project.
 *
 * Copyright (C) 2005  EPITA Research and Development Laboratory.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <fstream>
#include "checker.hh"
#include "production.hh"
#include "grammar.hh"
#include "graphic.hh"

void	Graphic::makeGraphic(Grammar& g)
{
  g.forEachProduction(*this, &Graphic::prodToSh);
}

// Output a shell script, that contain production information
// and a dot representation of attributes
int	Graphic::prodToSh(Production& p)
{
  using namespace std;
  static unsigned cpt = 0;
  float pos[2] = {0., 0.};

  stringstream oss;
  oss << "/tmp/out_" << cpt << ".sh";
  ofstream f(oss.rdbuf()->str().c_str());
  cpt++;

  f << "PROD_NAME=\"";
  dotPrint(f, p);
  f << "\"" << std::endl
    << "get_production() {" << std::endl
    << "  cat <<PROD" << std::endl;
  p.printEvalRules(f);
  f << "PROD" << std::endl
    << "}" << std::endl;

  f << "get_dot() {" << std::endl
    << "  cat <<DOT" << std::endl
    << "digraph g" << cpt << " {" << std::endl
    << "  bgcolor = \"#EEEEEE\"" << std::endl
    << "  splines = true" << std::endl;

  AttrNodeIter it;
  for (it = p.anl_.begin(); it != p.anl_.end(); ++it)
    {
      AttrNode& n = it->second;
      f << "  node n" << n.getUid() + cpt
	<< " [label=\"" << n.getAttr()->getAttrName() << "\","
	<< " pos=\"" << (n.isUp() ? pos[0] : pos[1])
	<< (n.isUp() ? ",1.0\"]" : ",0.0\"]") << std::endl;

      for (unsigned i = 0; i < n.nbOut_; i++)
	f << "  n" << n.getUid() + cpt << " -> n"
	  << n.out_[i] + cpt << std::endl;

      pos[n.isUp() ? 0 : 1] += 1.5;
    }

  f << "}" << std::endl
    << "DOT" << std::endl
    << "}" << std::endl;
  return 0;
}
