/*
 * production.hh: this file is a part of sdf-attribute from the
 *                Transformers project.
 *
 * Copyright (C) 2005  EPITA Research and Development Laboratory.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef PRODUCTION_HH_
# define PRODUCTION_HH_

# include "evalrule.hh"
# include "graph.hh"

class Production
{
public:
  Production() : flag(false) {}

  std::string name;

  typedef std::vector<Production*> ChildrenNodeList;
  ChildrenNodeList childrenNode;
  ChildrenNodeList parentsNode;

  /*
  ** Evaluation rules.
  */
  typedef std::vector<EvalRule> EvalRuleList;
  std::vector<EvalRule> evalRules;

  bool has_root_attribute(const Attribute& attr) const;
  EvalRule& get_root_attribute(const Attribute& attr);
  bool has_attribute(const std::string& symbol, const Attribute& attr) const;

  EvalRule& addRule(const Attribute& copy);

  /*
  ** Non-terminals and labels.
  */
  struct NT
  {
    int index;
    std::string symbol;
    std::string label;
  };
  typedef std::vector<NT> NonTerminalList;
  std::vector<NT> nonTerminals;

  const std::string& labelToSymbol(const std::string label);
  std::vector<std::string> symbolToLabelList(const std::string symbol);

  /*
  ** Dependency graphs.
  */
  AttrNodeList anl_;
  AttrNodeList dAnl_;

  /*
  ** Other handy attributes.
  */

  bool flag; /* whether or not this node has been visited */
  ATerm aterm; /* original aterm: prod([child], sort, [attrs])) */
  ATermList attr_list; /* [<attribute list>] */
  int iprod; /* label(_, iprod) */

  // Debug functions.
  friend std::ostream& operator<< (std::ostream& os, const Production& p);
  friend std::ostream& dotPrint (std::ostream& os, const Production& p);
  void print() const;
  void printEvalRules(std::ostream& os) const;

  // Get the resulting aterm, for the final printing.
  ATerm getAterm() const;
};

#endif /* !PRODUCTION_HH_ */
