#include <srts/stratego.h>
#include "native-lazy.h"

typedef ATerm (*strategy_t) (ATerm);

typedef struct _formula_list_t {
  struct _formula_node_t *cur;
  struct _formula_list_t *next;
} formula_list_t;

typedef struct _value_list_t {
  struct _value_node_t *cur;
  struct _value_list_t *next;
} value_list_t;

typedef struct _formula_node_t {
  strategy_t strategy;
  ATerm input;
} formula_node_t;

typedef struct _value_node_t {
  struct _formula_list_t *formulas;
  ATerm value;
} value_node_t;

static Symbol invalid_sym;
static ATerm invalid_term;

static void init_module_constant_terms(void)
{
  ATprotect(&(invalid_term));
  invalid_term = (ATerm) ATmakeAppl(invalid_sym);
}

static void init_module_constructors(void)
{
  invalid_sym = ATmakeSymbol("invalid", 0, ATfalse);
  ATprotectSymbol(invalid_sym);
}

#include <srts/init-stratego-module.h>

static void compute(value_node_t *v)
{
  formula_list_t *l = v->formulas, *oldl;
  int sp = stack_ptr;

  while (l) {
    if ((PushChoice() == 0)) {
      ATerm res = l->cur->strategy(l->cur->input);
      LocalPopChoice(sp);
      if (res != NULL) {
	if (!(match_cons(res, invalid_sym))) {
	  v->value = res;
	  ATprotect(&(v->value));
	  for (l = v->formulas; l != NULL;) {
	    ATunprotect(&(l->cur->input));
	    oldl = l;
	    l = l->next;
	    free(oldl);
	  }
	  v->formulas = NULL;
	  return ;
	}
      }
    }
    l = l->next;
  }
  v->value = invalid_term;
  ATprotect(&(v->value));
}

ATerm LAZY_getv(ATerm t)
{
  value_node_t *value;

  if (!ATisInt(t)) {
    ATfprintf(stderr, "LAZY_formula: reference not an integer: %t\n", t);
    _fail(t);
  }

  value = (value_node_t *)AT_getInt(t);

  if (value->value == NULL)
    compute(value);

  if ((match_cons(value->value, invalid_sym))) {
    _fail(value->value);
  }

  return value->value;
}

ATerm LAZY_new_value(void)
{
  value_node_t *node = malloc(sizeof(value_node_t));

  if (node == NULL) {
    ATfprintf(stderr, "LAZY_new_value: allocation failed.\n");
    _fail(NULL);
  }

  node->value = NULL;
  node->formulas = NULL;

  return (ATerm) ATmakeInt((int) node);
}

ATerm LAZY_formula(ATerm s(ATerm),
		   ATerm result, ATerm input)
{
  formula_node_t *formula = malloc(sizeof(formula_node_t));
  value_node_t *value;
  formula_list_t *list;

  if (formula == NULL) {
    ATfprintf(stderr, "LAZY_formula: allocation failed.\n");
    _fail(input);
  }

  ATprotect(&(formula->input));

  formula->strategy = s;
  formula->input = input;

  if (!ATisInt(result)) {
    ATfprintf(stderr, "LAZY_formula: result reference not an integer: %t\n",
	      result);
    _fail(result);
  }

  value = (value_node_t *)AT_getInt(result);

  list = value->formulas;
  value->formulas = malloc(sizeof(value_list_t));
  if (value->formulas == NULL) {
    ATfprintf(stderr, "LAZY_formula: allocation failed.\n");
    value->formulas = list;
    _fail(result);
  }
  value->formulas->cur = formula;
  value->formulas->next = list;

  return input;
}
